@@ -22,12 +22,13 @@ import (
 	awsprovider "sigs.k8s.io/cluster-api-provider-aws/pkg/apis/awsprovider/v1beta1"
 	"sigs.k8s.io/controller-runtime/pkg/client/fake"
 
-	machineapi "github.com/openshift/cluster-api/pkg/apis/machine/v1beta1"
+	machineapi "github.com/openshift/machine-api-operator/pkg/apis/machine/v1beta1"
 
-	"github.com/openshift/hive/pkg/apis"
-	hivev1 "github.com/openshift/hive/pkg/apis/hive/v1"
-	awshivev1 "github.com/openshift/hive/pkg/apis/hive/v1/aws"
+	"github.com/openshift/hive/apis"
+	hivev1 "github.com/openshift/hive/apis/hive/v1"
+	awshivev1 "github.com/openshift/hive/apis/hive/v1/aws"
 	mockaws "github.com/openshift/hive/pkg/awsclient/mock"
+	controllerutils "github.com/openshift/hive/pkg/controller/utils"
 )
 
 func TestAWSActuator(t *testing.T) {
@@ -97,12 +98,23 @@ func TestAWSActuator(t *testing.T) {
 				func() *hivev1.MachinePool {
 					pool := testMachinePool()
 					pool.Spec.Platform.AWS.Zones = []string{"zone1", "zone2", "zone3"}
-					pool.Spec.Platform.AWS.Subnets = []string{"subnet-zone1", "subnet-zone2", "subnet-zone3"}
+					pool.Spec.Platform.AWS.Subnets = []string{"subnet-zone1", "subnet-zone2", "subnet-zone3",
+						"pubSubnet-zone1", "pubSubnet-zone2", "pubSubnet-zone3"}
 					return pool
 				}(),
 			},
 			mockAWSClient: func(client *mockaws.MockClient) {
-				mockDescribeSubnets(client, []string{"zone1", "zone2", "zone3"}, []string{"subnet-zone1", "subnet-zone2", "subnet-zone3"})
+				mockDescribeSubnets(client, []string{"zone1", "zone2", "zone3"},
+					[]string{"subnet-zone1", "subnet-zone2", "subnet-zone3"},
+					[]string{"pubSubnet-zone1", "pubSubnet-zone2", "pubSubnet-zone3"}, "vpc-1")
+				mockDescribeRouteTables(client, map[string]bool{
+					"subnet-zone1":    false,
+					"subnet-zone2":    false,
+					"subnet-zone3":    false,
+					"pubSubnet-zone1": true,
+					"pubSubnet-zone2": true,
+					"pubSubnet-zone3": true,
+				}, "vpc-1")
 			},
 			expectedMachineSetReplicas: map[string]int64{
 				generateAWSMachineSetName("zone1"): 1,
@@ -146,7 +158,7 @@ func TestAWSActuator(t *testing.T) {
 			},
 		},
 		{
-			name:              "more than one subnet for availability zone",
+			name:              "more than one private subnet for availability zone",
 			clusterDeployment: testClusterDeployment(),
 			poolName:          testMachinePool().Name,
 			existing: []runtime.Object{
@@ -158,7 +170,13 @@ func TestAWSActuator(t *testing.T) {
 				}(),
 			},
 			mockAWSClient: func(client *mockaws.MockClient) {
-				mockDescribeSubnets(client, []string{"zone1", "zone1", "zone2"}, []string{"subnet-zone1", "subnet-zone2", "subnet-zone3"})
+				mockDescribeSubnets(client, []string{"zone1", "zone1", "zone2"},
+					[]string{"subnet-zone1", "subnet-zone2", "subnet-zone3"}, []string{}, "vpc-1")
+				mockDescribeRouteTables(client, map[string]bool{
+					"subnet-zone1": false,
+					"subnet-zone2": false,
+					"subnet-zone3": false,
+				}, "vpc-1")
 			},
 			expectedErr: true,
 			expectedCondition: &hivev1.MachinePoolCondition{
@@ -168,7 +186,7 @@ func TestAWSActuator(t *testing.T) {
 			},
 		},
 		{
-			name:              "no subnet for availability zone",
+			name:              "no private subnet for availability zone",
 			clusterDeployment: testClusterDeployment(),
 			poolName:          testMachinePool().Name,
 			existing: []runtime.Object{
@@ -180,7 +198,12 @@ func TestAWSActuator(t *testing.T) {
 				}(),
 			},
 			mockAWSClient: func(client *mockaws.MockClient) {
-				mockDescribeSubnets(client, []string{"zone1", "zone2", "zone3"}, []string{"subnet-zone1", "subnet-zone2"})
+				mockDescribeSubnets(client, []string{"zone1", "zone2", "zone3"},
+					[]string{"subnet-zone1", "subnet-zone2"}, []string{}, "vpc-1")
+				mockDescribeRouteTables(client, map[string]bool{
+					"subnet-zone1": false,
+					"subnet-zone2": false,
+				}, "vpc-1")
 			},
 			expectedErr: true,
 			expectedCondition: &hivev1.MachinePoolCondition{
@@ -189,6 +212,34 @@ func TestAWSActuator(t *testing.T) {
 				Reason: "NoSubnetForAvailabilityZone",
 			},
 		},
+		{
+			name:              "no public subnet for availability zone and private subnet",
+			clusterDeployment: testClusterDeployment(),
+			poolName:          testMachinePool().Name,
+			existing: []runtime.Object{
+				func() *hivev1.MachinePool {
+					pool := testMachinePool()
+					pool.Spec.Platform.AWS.Zones = []string{"zone1", "zone2"}
+					pool.Spec.Platform.AWS.Subnets = []string{"subnet-zone1", "subnet-zone2", "pubSubnet-zone1"}
+					return pool
+				}(),
+			},
+			mockAWSClient: func(client *mockaws.MockClient) {
+				mockDescribeSubnets(client, []string{"zone1", "zone2"},
+					[]string{"subnet-zone1", "subnet-zone2"}, []string{"pubSubnet-zone1"}, "vpc-1")
+				mockDescribeRouteTables(client, map[string]bool{
+					"subnet-zone1":    false,
+					"subnet-zone2":    false,
+					"pubSubnet-zone1": true,
+				}, "vpc-1")
+			},
+			expectedErr: true,
+			expectedCondition: &hivev1.MachinePoolCondition{
+				Type:   hivev1.InvalidSubnetsMachinePoolCondition,
+				Status: corev1.ConditionTrue,
+				Reason: "InsufficientPublicSubnets",
+			},
+		},
 		{
 			name:              "supported spot market options",
 			clusterDeployment: withClusterVersion(testClusterDeployment(), "4.5.0"),
@@ -291,14 +342,11 @@ func TestAWSActuator(t *testing.T) {
 				validateAWSMachineSets(t, generatedMachineSets, test.expectedMachineSetReplicas, test.expectedSubnetIDInMachineSet)
 			}
 			if test.expectedCondition != nil {
-				for _, cond := range pool.Status.Conditions {
-					assert.Equal(t, cond.Type, test.expectedCondition.Type)
-					assert.Equal(t, cond.Status, test.expectedCondition.Status)
-					assert.Equal(t, cond.Reason, test.expectedCondition.Reason)
+				cond := controllerutils.FindMachinePoolCondition(pool.Status.Conditions, test.expectedCondition.Type)
+				if assert.NotNilf(t, cond, "did not find expected condition type: %v", test.expectedCondition.Type) {
+					assert.Equal(t, test.expectedCondition.Status, cond.Status, "condition found with unexpected status")
+					assert.Equal(t, test.expectedCondition.Reason, cond.Reason, "condition found with unexpected reason")
 				}
-			} else {
-				// Assuming if you didn't expect a condition, there shouldn't be any.
-				assert.Equal(t, 0, len(pool.Status.Conditions))
 			}
 		})
 	}
@@ -386,19 +434,30 @@ func mockDescribeAvailabilityZones(client *mockaws.MockClient, zones []string) {
 	client.EXPECT().DescribeAvailabilityZones(input).Return(output, nil)
 }
 
-func mockDescribeSubnets(client *mockaws.MockClient, zones []string, subnetIDs []string) {
-	idPointers := make([]*string, 0, len(subnetIDs))
-	for _, id := range subnetIDs {
+func mockDescribeSubnets(client *mockaws.MockClient, zones []string, privateSubnetIDs []string, pubSubnetIDs []string, vpcID string) {
+	idPointers := make([]*string, 0, len(privateSubnetIDs)+len(pubSubnetIDs))
+	for _, id := range privateSubnetIDs {
+		idPointers = append(idPointers, aws.String(id))
+	}
+	for _, id := range pubSubnetIDs {
 		idPointers = append(idPointers, aws.String(id))
 	}
 	input := &ec2.DescribeSubnetsInput{
 		SubnetIds: idPointers,
 	}
-	subnets := make([]*ec2.Subnet, len(subnetIDs))
-	for i := range subnetIDs {
+	subnets := make([]*ec2.Subnet, len(privateSubnetIDs)+len(pubSubnetIDs))
+	for i := range privateSubnetIDs {
 		subnets[i] = &ec2.Subnet{
-			SubnetId:         &subnetIDs[i],
+			SubnetId:         &privateSubnetIDs[i],
 			AvailabilityZone: &zones[i],
+			VpcId:            &vpcID,
+		}
+	}
+	for i := range pubSubnetIDs {
+		subnets[len(privateSubnetIDs)+i] = &ec2.Subnet{
+			SubnetId:         &pubSubnetIDs[i],
+			AvailabilityZone: &zones[i],
+			VpcId:            &vpcID,
 		}
 	}
 	output := &ec2.DescribeSubnetsOutput{
@@ -418,6 +477,60 @@ func mockDescribeMissingSubnets(client *mockaws.MockClient, subnetIDs []string)
 	client.EXPECT().DescribeSubnets(input).Return(nil, fmt.Errorf("InvalidSubnets"))
 }
 
+func mockDescribeRouteTables(client *mockaws.MockClient, subnets map[string]bool, vpc string) {
+	input := &ec2.DescribeRouteTablesInput{
+		Filters: []*ec2.Filter{{
+			Name:   aws.String("vpc-id"),
+			Values: []*string{aws.String(vpc)},
+		}},
+	}
+
+	output := &ec2.DescribeRouteTablesOutput{
+		RouteTables: constructRouteTables(subnets),
+	}
+
+	client.EXPECT().DescribeRouteTables(input).Return(output, nil)
+}
+
+// Takes a list of subnets with bool indicating if the corresponding subnet is public
+func constructRouteTables(subnets map[string]bool) (routeTablesOut []*ec2.RouteTable) {
+	routeTablesOut = append(routeTablesOut,
+		&ec2.RouteTable{
+			Associations: []*ec2.RouteTableAssociation{{Main: aws.Bool(true)}},
+			Routes: []*ec2.Route{{
+				DestinationCidrBlock: aws.String("0.0.0.0/0"),
+				GatewayId:            aws.String("igw-main"),
+			}},
+		})
+
+	for subnetID := range subnets {
+		routeTablesOut = append(
+			routeTablesOut,
+			constructRouteTable(
+				subnetID,
+				subnets[subnetID],
+			),
+		)
+	}
+	return
+}
+
+func constructRouteTable(subnetID string, public bool) *ec2.RouteTable {
+	var gatewayID string
+	if public {
+		gatewayID = "igw-" + subnetID[len(subnetID)-8:8]
+	} else {
+		gatewayID = "vgw-" + subnetID[len(subnetID)-8:8]
+	}
+	return &ec2.RouteTable{
+		Associations: []*ec2.RouteTableAssociation{{SubnetId: aws.String(subnetID)}},
+		Routes: []*ec2.Route{{
+			DestinationCidrBlock: aws.String("0.0.0.0/0"),
+			GatewayId:            aws.String(gatewayID),
+		}},
+	}
+}
+
 func generateAWSMachineSetName(zone string) string {
 	return fmt.Sprintf("%s-%s-%s", testInfraID, testPoolName, zone)
 }
